import { _decorator, Component, Node, Prefab, Size, view, instantiate, UITransform, v3 } from 'cc';
import { ShowNode } from './LoopList';
const { ccclass, property } = _decorator;


@ccclass('ItemCell')
export default class ItemCell<T> extends Component {
    /**显示节点 */
    public showNode: Node = null;
    /**尺寸 */
    public size: Size = null;
    /**数据 */
    public data: T = null;

    /**
     * 初始化
     * @param data 
     * @param size 
     */
    public init(data: T, size: Size) {
        let uiTransform = this.getComponent(UITransform);
        uiTransform.width = size.width;
        uiTransform.height = size.height;
        this.data = data;
        this.size = size;
    }

    /**
     * 
     * 更新节点显示
     * @param isShow 是否显示
     * @param prefab 需要显示的节点的预制件（也可以是节点，或者预制件url,生成方式自由改动）
     * @param cacheNode 对象池中的显示节点
     */
    updateShow(isShow: boolean, prefab: Prefab, cacheNode: Node) {
        if (isShow) {
            //当前没有显示节点
            if (!this.showNode) {
                //如果是临时对象池还有节点则重用
                if (cacheNode) {
                    this.showNode = cacheNode;
                } else {//否则使用传入的预制件进行生成
                    this.showNode = instantiate(prefab);
                }
            }
            this.showNode.setParent(this.node);
            this.showNode.position = v3();
            this.showNode.getComponent(ShowNode).initShow(this.data);
        }
    }

    recircle() {
        if (this.showNode) {
            let node = this.showNode;
            this.showNode.removeFromParent();
            this.showNode = null;
            return node;
        }
    }
}


