import { _decorator, Component, Node, Prefab, Size, view, instantiate, ScrollView, UITransform } from 'cc';
import ItemCell from './ItemCell';
const { ccclass, property } = _decorator;

export class ShowNode extends Component {
    /**初始化显示 */
    initShow(data: any) { }
}

@ccclass('LoopList')
export default class LoopList<T> extends Component {
    /**滚动组件 */
    @property(ScrollView)
    public scrollView: ScrollView = null;
    /**数据 */
    public data: T[] = [];
    /**item节点们 */
    public itemCells: ItemCell<T>[] = [];
    /**通用尺寸 */
    public size: Size = null;
    /**预制件 */
    public prefab: Prefab = null;
    /**显示节点的池子 */
    public showNodePool: Node[] = [];
    /**边缘数据 */
    public sideArr: { viewLeftSide?: number, viewRightSide?: number, viewBottomSide?: number, viewTipSide?: number, } = {};
    /**刷新显示的间隔次数 */
    public updateInv: number = 0;

    onLoad() {
        //显示区域左边缘
        let viewLeftSide = this.scrollView.view.node.worldPosition.x - (this.scrollView.view.width * this.scrollView.view.anchorX);
        //显示区域右边缘
        let viewRightSide = this.scrollView.view.node.worldPosition.x + (this.scrollView.view.width * (1 - this.scrollView.view.anchorX));
        //显示区域下边缘
        let viewBottomSide = this.scrollView.view.node.worldPosition.y - (this.scrollView.view.height * (1 - this.scrollView.view.anchorY));
        //显示区域上边缘
        let viewTipSide = this.scrollView.view.node.worldPosition.y + (this.scrollView.view.height * this.scrollView.view.anchorY);
        this.sideArr = {
            viewLeftSide, viewRightSide, viewBottomSide, viewTipSide
        };

        this.scrollView.node.on(ScrollView.EventType.SCROLLING, () => {
            this.updateInv--;
            if (this.updateInv <= 0) {
                this.updateInv = 3;
                this.updateNodes();
            }
        })
    }

    /**
     * 初始化
     * @param data 数据组
     * @param prefab 预制件
     * @param size 通用尺寸
     */
    public initData(data: T[], prefab: Prefab, size?: Size) {
        this.data = data;
        this.size = size;
        this.prefab = prefab;
        for (let i = 0, len = data.length; i < len; ++i) {
            let itemCell = this.itemCells[i];
            if (!itemCell) {
                let node = new Node('ItemCell');
                node.addComponent(UITransform);
                node.setParent(this.scrollView.content);
                itemCell = this.itemCells[i] = node.addComponent(ItemCell<T>);
            }
            let data = this.data[i] as any;
            let cellSize = data.size ? data.size : size;
            itemCell.init(data, cellSize);
        }
        for (let i = data.length, len = this.itemCells.length; i < len; ++i) {
            this.recircle(this.itemCells[i].recircle());
            this.itemCells[i] = null;
        }
        requestAnimationFrame(() => {
            this.updateNodes();
        })
    }


    /**更新节点的显示 */
    public updateNodes() {
        for (let i = 0, len = this.itemCells.length; i < len; ++i) {
            let itemCell = this.itemCells[i];
            if (itemCell) {
                let isShow = this.checkInView(itemCell);
                if (isShow) {
                    itemCell.updateShow(isShow, this.prefab, this.showNodePool.length > 0 ? this.showNodePool.pop() : null);
                } else {
                    this.recircle(this.itemCells[i].recircle());
                }
            }
        }
    }


    /**
     * 检查是否在屏幕内
     * @param node 
     * @returns 
     */
    checkInView(itemCell: ItemCell<T>) {
        //节点右侧边缘在屏幕左侧外面或者节点左边缘在屏幕右侧外面 则不在显示区域
        let horizontalOutside = itemCell.node.worldPosition.x + (itemCell.size.x * 0.5) < this.sideArr.viewLeftSide
            || itemCell.node.worldPosition.x - (itemCell.size.x * 0.5) > this.sideArr.viewRightSide;

        let verticalOutside = false;
        if (!horizontalOutside) {
            //节点上边缘在屏幕下方外面或者节点下边缘在屏幕上方外面 则不在显示区域
            verticalOutside = itemCell.node.worldPosition.y + (itemCell.size.y * 0.5) < this.sideArr.viewBottomSide
                || itemCell.node.worldPosition.y - (itemCell.size.y * 0.5) > this.sideArr.viewTipSide;
        }
        return !horizontalOutside && !verticalOutside;
    }

    /**
     * 回收显示节点
     * @param node 
     */
    recircle(node: Node) {
        node && this.showNodePool.push(node);
    }

}


